@@ -533,7 +533,7 @@ def make_v0(apps, apks, repodir, repodict, requestsdict, fdroid_signing_key_fing
                 old_permissions = set()
                 sorted_permissions = sorted(apk['uses-permission'])
                 for perm in sorted_permissions:
-                    perm_name = perm.name
+                    perm_name = perm[0]
                     if perm_name.startswith("android.permission."):
                         perm_name = perm_name[19:]
                     old_permissions.add(perm_name)
@@ -541,15 +541,15 @@ def make_v0(apps, apks, repodir, repodict, requestsdict, fdroid_signing_key_fing
 
                 for permission in sorted_permissions:
                     permel = doc.createElement('uses-permission')
-                    permel.setAttribute('name', permission.name)
-                    if permission.maxSdkVersion is not None:
-                        permel.setAttribute('maxSdkVersion', '%d' % permission.maxSdkVersion)
+                    permel.setAttribute('name', permission[0])
+                    if permission[1] is not None:
+                        permel.setAttribute('maxSdkVersion', '%d' % permission[1])
                         apkel.appendChild(permel)
                 for permission_sdk_23 in sorted(apk['uses-permission-sdk-23']):
                     permel = doc.createElement('uses-permission-sdk-23')
-                    permel.setAttribute('name', permission_sdk_23.name)
-                    if permission_sdk_23.maxSdkVersion is not None:
-                        permel.setAttribute('maxSdkVersion', '%d' % permission_sdk_23.maxSdkVersion)
+                    permel.setAttribute('name', permission_sdk_23[0])
+                    if permission_sdk_23[1] is not None:
+                        permel.setAttribute('maxSdkVersion', '%d' % permission_sdk_23[1])
                         apkel.appendChild(permel)
                 if 'nativecode' in apk:
                     addElement('nativecode', ','.join(sorted(apk['nativecode'])), doc, apkel)
@@ -620,7 +620,8 @@ def extract_pubkey():
     if 'repo_pubkey' in common.config:
         pubkey = unhexlify(common.config['repo_pubkey'])
     else:
-        env_vars = {'FDROID_KEY_STORE_PASS': common.config['keystorepass']}
+        env_vars = {'LC_ALL': 'C.UTF-8',
+                    'FDROID_KEY_STORE_PASS': common.config['keystorepass']}
         p = FDroidPopenBytes([common.config['keytool'], '-exportcert',
                               '-alias', common.config['repo_keyalias'],
                               '-keystore', common.config['keystore'],
@@ -685,16 +686,19 @@ def get_mirror_service_urls(url):
     return urls
 
 
-def download_repo_index(url_str, etag=None, verify_fingerprint=True):
-    """
-    Downloads the repository index from the given :param url_str
-    and verifies the repository's fingerprint if :param verify_fingerprint is not False.
+def download_repo_index(url_str, etag=None, verify_fingerprint=True, timeout=600):
+    """Downloads and verifies index file, then returns its data.
+
+    Downloads the repository index from the given :param url_str and
+    verifies the repository's fingerprint if :param verify_fingerprint
+    is not False.
 
     :raises: VerificationException() if the repository could not be verified
 
     :return: A tuple consisting of:
         - The index in JSON format or None if the index did not change
         - The new eTag as returned by the HTTP request
+
     """
     url = urllib.parse.urlsplit(url_str)
 
@@ -706,38 +710,37 @@ def download_repo_index(url_str, etag=None, verify_fingerprint=True):
         fingerprint = query['fingerprint'][0]
 
     url = urllib.parse.SplitResult(url.scheme, url.netloc, url.path + '/index-v1.jar', '', '')
-    download, new_etag = net.http_get(url.geturl(), etag)
+    download, new_etag = net.http_get(url.geturl(), etag, timeout)
 
     if download is None:
         return None, new_etag
 
     with tempfile.NamedTemporaryFile() as fp:
-        # write and open JAR file
         fp.write(download)
-        jar = zipfile.ZipFile(fp)
-
-        # verify that the JAR signature is valid
-        logging.debug(_('Verifying index signature:'))
-        common.verify_jar_signature(fp.name)
-
-        # get public key and its fingerprint from JAR
-        public_key, public_key_fingerprint = get_public_key_from_jar(jar)
-
-        # compare the fingerprint if verify_fingerprint is True
-        if verify_fingerprint and fingerprint.upper() != public_key_fingerprint:
-            raise VerificationException(_("The repository's fingerprint does not match."))
-
-        # load repository index from JSON
-        index = json.loads(jar.read('index-v1.json').decode("utf-8"))
-        index["repo"]["pubkey"] = hexlify(public_key).decode("utf-8")
+        index, public_key, public_key_fingerprint = get_index_from_jar(fp.name, fingerprint)
+        index["repo"]["pubkey"] = hexlify(public_key).decode()
         index["repo"]["fingerprint"] = public_key_fingerprint
-
-        # turn the apps into App objects
         index["apps"] = [metadata.App(app) for app in index["apps"]]
-
         return index, new_etag
 
 
+def get_index_from_jar(jarfile, fingerprint=None):
+    """Returns the data, public key, and fingerprint from index-v1.jar
+
+    :raises: VerificationException() if the repository could not be verified
+    """
+
+    logging.debug(_('Verifying index signature:'))
+    common.verify_jar_signature(jarfile)
+    with zipfile.ZipFile(jarfile) as jar:
+        public_key, public_key_fingerprint = get_public_key_from_jar(jar)
+        if fingerprint is not None:
+            if fingerprint.upper() != public_key_fingerprint:
+                raise VerificationException(_("The repository's fingerprint does not match."))
+        data = json.loads(jar.read('index-v1.json').decode())
+        return data, public_key, public_key_fingerprint
+
+
 def get_public_key_from_jar(jar):
     """
     Get the public key and its fingerprint from a JAR file.
@@ -748,7 +751,7 @@ def get_public_key_from_jar(jar):
     :return: the public key from the jar and its fingerprint
     """
     # extract certificate from jar
-    certs = [n for n in jar.namelist() if common.CERT_PATH_REGEX.match(n)]
+    certs = [n for n in jar.namelist() if common.SIGNATURE_BLOCK_FILE_REGEX.match(n)]
     if len(certs) < 1:
         raise VerificationException(_("Found no signing certificates for repository."))
     if len(certs) > 1:
