@@ -3,6 +3,7 @@
 # metadata.py - part of the FDroid server tools
 # Copyright (C) 2013, Ciaran Gultnieks, ciaran@ciarang.com
 # Copyright (C) 2013-2014 Daniel Martí <mvdan@mvdan.cc>
+# Copyright (C) 2017-2018 Michael Pöhn <michael.poehn@fsfe.org>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as published by
@@ -27,13 +28,6 @@ import textwrap
 import io
 import yaml
 from collections import OrderedDict
-# use libyaml if it is available
-try:
-    from yaml import CLoader
-    YamlLoader = CLoader
-except ImportError:
-    from yaml import Loader
-    YamlLoader = Loader
 
 import fdroidserver.common
 from fdroidserver import _
@@ -99,6 +93,57 @@ app_fields = set([
     'builds',    # For formats that do builds as a list
 ])
 
+yaml_app_field_order = [
+    'Disabled',
+    'AntiFeatures',
+    'Provides',
+    'Categories',
+    'License',
+    'AuthorName',
+    'AuthorEmail',
+    'AuthorWebSite',
+    'WebSite',
+    'SourceCode',
+    'IssueTracker',
+    'Translation',
+    'Changelog',
+    'Donate',
+    'FlattrID',
+    'LiberapayID',
+    'Bitcoin',
+    'Litecoin',
+    '\n',
+    'Name',
+    'AutoName',
+    'Summary',
+    'Description',
+    '\n',
+    'RequiresRoot',
+    '\n',
+    'RepoType',
+    'Repo',
+    'Binaries',
+    '\n',
+    'Builds',
+    '\n',
+    'MaintainerNotes',
+    '\n',
+    'ArchivePolicy',
+    'AutoUpdateMode',
+    'UpdateCheckMode',
+    'UpdateCheckIgnore',
+    'VercodeOperation',
+    'UpdateCheckName',
+    'UpdateCheckData',
+    'CurrentVersion',
+    'CurrentVersionCode',
+    '\n',
+    'NoSourceSince',
+]
+
+
+yaml_app_fields = [x for x in yaml_app_field_order if x != '\n']
+
 
 class App(dict):
 
@@ -111,7 +156,7 @@ class App(dict):
         self.Disabled = None
         self.AntiFeatures = []
         self.Provides = None
-        self.Categories = ['None']
+        self.Categories = []
         self.License = 'Unknown'
         self.AuthorName = None
         self.AuthorEmail = None
@@ -183,16 +228,15 @@ TYPE_LIST = 4
 TYPE_SCRIPT = 5
 TYPE_MULTILINE = 6
 TYPE_BUILD = 7
-TYPE_BUILD_V2 = 8
-TYPE_INT = 9
+TYPE_INT = 8
 
 fieldtypes = {
     'Description': TYPE_MULTILINE,
     'MaintainerNotes': TYPE_MULTILINE,
     'Categories': TYPE_LIST,
     'AntiFeatures': TYPE_LIST,
-    'BuildVersion': TYPE_BUILD,
-    'Build': TYPE_BUILD_V2,
+    'Build': TYPE_BUILD,
+    'BuildVersion': TYPE_OBSOLETE,
     'UseBuilt': TYPE_OBSOLETE,
 }
 
@@ -249,7 +293,7 @@ class Build(dict):
 
     def __init__(self, copydict=None):
         super().__init__()
-        self.disable = False
+        self.disable = ''
         self.commit = None
         self.timeout = None
         self.subdir = None
@@ -429,7 +473,7 @@ valuetypes = {
                    ["ArchivePolicy"]),
 
     FieldValidator("Anti-Feature",
-                   r'^(Ads|Tracking|NonFreeNet|NonFreeDep|NonFreeAdd|UpstreamNonFree|NonFreeAssets|KnownVuln|ApplicationDebuggable)$',
+                   r'^(Ads|Tracking|NonFreeNet|NonFreeDep|NonFreeAdd|UpstreamNonFree|NonFreeAssets|KnownVuln|ApplicationDebuggable|NoSourceSince)$',
                    ["AntiFeatures"]),
 
     FieldValidator("Auto Update Mode",
@@ -662,7 +706,7 @@ def parse_srclib(metadatapath):
     if not os.path.exists(metadatapath):
         return thisinfo
 
-    metafile = open(metadatapath, "r", encoding='utf-8')
+    metafile = open(metadatapath, "r")
 
     n = 0
     for line in metafile:
@@ -760,6 +804,9 @@ def read_metadata(xref=True, check_vcs=[], refresh=True, sort_by_time=False):
         if metadatapath == '.fdroid.txt':
             warn_or_exception(_('.fdroid.txt is not supported!  Convert to .fdroid.yml or .fdroid.json.'))
         appid, _ignored = fdroidserver.common.get_extension(os.path.basename(metadatapath))
+        if appid != '.fdroid' and not fdroidserver.common.is_valid_package_name(appid):
+            warn_or_exception(_("{appid} from {path} is not a valid Java Package Name!")
+                              .format(appid=appid, path=metadatapath))
         if appid in apps:
             warn_or_exception(_("Found multiple metadata files for {appid}")
                               .format(appid=appid))
@@ -854,12 +901,14 @@ def post_metadata_parse(app):
     if 'flavours' in app and app['flavours'] == [True]:
         app['flavours'] = 'yes'
 
-    if isinstance(app.Categories, str):
-        app.Categories = [app.Categories]
-    elif app.Categories is None:
-        app.Categories = ['None']
-    else:
-        app.Categories = [str(i) for i in app.Categories]
+    for field, fieldtype in fieldtypes.items():
+        if fieldtype != TYPE_LIST:
+            continue
+        value = app.get(field)
+        if isinstance(value, str):
+            app[field] = [value, ]
+        elif value is not None:
+            app[field] = [str(i) for i in value]
 
     def _yaml_bool_unmapable(v):
         return v in (True, False, [True], [False])
@@ -874,7 +923,7 @@ def post_metadata_parse(app):
         elif v == [False]:
             return ['no']
 
-    _bool_allowed = ('disable', 'maven', 'buildozer')
+    _bool_allowed = ('maven', 'buildozer')
 
     builds = []
     if 'builds' in app:
@@ -965,7 +1014,7 @@ def parse_metadata(metadatapath, check_vcs=False, refresh=True):
     else:
         app.id = name
 
-    with open(metadatapath, 'r', encoding='utf-8') as mf:
+    with open(metadatapath, 'r') as mf:
         if ext == 'txt':
             parse_txt_metadata(mf, app)
         elif ext == 'json':
@@ -1021,13 +1070,50 @@ def parse_json_metadata(mf, app):
 
 
 def parse_yaml_metadata(mf, app):
-    yamldata = yaml.load(mf, Loader=YamlLoader)
+    yamldata = yaml.safe_load(mf)
+
     if yamldata:
+        for field in yamldata:
+            if field not in yaml_app_fields:
+                warn_or_exception(_("Unrecognised app field '{fieldname}' "
+                                    "in '{path}'").format(fieldname=field,
+                                                          path=mf.name))
+        if yamldata.get('Builds', None):
+            for build in yamldata.get('Builds', []):
+                # put all build flag keywords into a set to avoid
+                # excessive looping action
+                build_flag_set = set()
+                for build_flag in build.keys():
+                    build_flag_set.add(build_flag)
+                for build_flag in build_flag_set:
+                    if build_flag not in build_flags:
+                        warn_or_exception(
+                            _("Unrecognised build flag '{build_flag}' "
+                              "in '{path}'").format(build_flag=build_flag,
+                                                    path=mf.name))
+        post_parse_yaml_metadata(yamldata)
         app.update(yamldata)
     return app
 
 
+def post_parse_yaml_metadata(yamldata):
+    """transform yaml metadata to our internal data format"""
+    for build in yamldata.get('Builds', []):
+        for flag in build.keys():
+            _flagtype = flagtype(flag)
+
+            # concatenate script flags into a single string if they are stored as list
+            if _flagtype is TYPE_SCRIPT:
+                if isinstance(build[flag], list):
+                    build[flag] = ' && '.join(build[flag])
+
+
 def write_yaml(mf, app):
+    """Write metadata in yaml format.
+
+    :param mf: active file discriptor for writing
+    :param app: app metadata to written to the yaml file
+    """
 
     # import rumael.yaml and check version
     try:
@@ -1073,10 +1159,17 @@ def write_yaml(mf, app):
             else:
                 return str(value)
         elif typ is TYPE_SCRIPT:
-            if len(value) > 50:
-                return ruamel.yaml.scalarstring.preserve_literal(value)
+            if type(value) == list:
+                if len(value) == 1:
+                    return value[0]
+                else:
+                    return value
             else:
-                return value
+                script_lines = value.split(' && ')
+                if len(script_lines) > 1:
+                    return script_lines
+                else:
+                    return value
         else:
             return value
 
@@ -1084,16 +1177,16 @@ def write_yaml(mf, app):
         cm = ruamel.yaml.comments.CommentedMap()
         insert_newline = False
         for field in yaml_app_field_order:
-            if field is '\n':
+            if field == '\n':
                 # next iteration will need to insert a newline
                 insert_newline = True
             else:
-                if app.get(field) or field is 'Builds':
+                if app.get(field) or field == 'Builds':
                     # .txt calls it 'builds' internally, everywhere else its 'Builds'
-                    if field is 'Builds':
+                    if field == 'Builds':
                         if app.get('builds'):
                             cm.update({field: _builds_to_yaml(app)})
-                    elif field is 'CurrentVersionCode':
+                    elif field == 'CurrentVersionCode':
                         cm.update({field: _field_to_yaml(TYPE_INT, getattr(app, field))})
                     else:
                         cm.update({field: _field_to_yaml(fieldtype(field), getattr(app, field))})
@@ -1108,17 +1201,15 @@ def write_yaml(mf, app):
         return cm
 
     def _builds_to_yaml(app):
-        fields = ['versionName', 'versionCode']
-        fields.extend(build_flags_order)
         builds = ruamel.yaml.comments.CommentedSeq()
         for build in app.builds:
             b = ruamel.yaml.comments.CommentedMap()
-            for field in fields:
-                if hasattr(build, field) and getattr(build, field):
-                    value = getattr(build, field)
+            for field in build_flags:
+                value = getattr(build, field)
+                if hasattr(build, field) and value:
                     if field == 'gradle' and value == ['off']:
                         value = [ruamel.yaml.scalarstring.SingleQuotedScalarString('off')]
-                    if field in ('disable', 'maven', 'buildozer'):
+                    if field in ('maven', 'buildozer'):
                         if value == 'no':
                             continue
                         elif value == 'yes':
@@ -1133,54 +1224,6 @@ def write_yaml(mf, app):
 
         return builds
 
-    yaml_app_field_order = [
-        'Disabled',
-        'AntiFeatures',
-        'Provides',
-        'Categories',
-        'License',
-        'AuthorName',
-        'AuthorEmail',
-        'AuthorWebSite',
-        'WebSite',
-        'SourceCode',
-        'IssueTracker',
-        'Translation',
-        'Changelog',
-        'Donate',
-        'FlattrID',
-        'LiberapayID',
-        'Bitcoin',
-        'Litecoin',
-        '\n',
-        'Name',
-        'AutoName',
-        'Summary',
-        'Description',
-        '\n',
-        'RequiresRoot',
-        '\n',
-        'RepoType',
-        'Repo',
-        'Binaries',
-        '\n',
-        'Builds',
-        '\n',
-        'MaintainerNotes',
-        '\n',
-        'ArchivePolicy',
-        'AutoUpdateMode',
-        'UpdateCheckMode',
-        'UpdateCheckIgnore',
-        'VercodeOperation',
-        'UpdateCheckName',
-        'UpdateCheckData',
-        'CurrentVersion',
-        'CurrentVersionCode',
-        '\n',
-        'NoSourceSince',
-    ]
-
     yaml_app = _app_to_yaml(app)
     ruamel.yaml.round_trip_dump(yaml_app, mf, indent=4, block_seq_indent=2)
 
@@ -1314,7 +1357,7 @@ def parse_txt_metadata(mf, app):
             f = f.replace(' ', '')
 
             ftype = fieldtype(f)
-            if ftype not in [TYPE_BUILD, TYPE_BUILD_V2]:
+            if ftype not in [TYPE_BUILD]:
                 add_comments(f)
             if ftype == TYPE_MULTILINE:
                 mode = 1
@@ -1326,15 +1369,6 @@ def parse_txt_metadata(mf, app):
             elif ftype == TYPE_LIST:
                 app[f] = split_list_values(v)
             elif ftype == TYPE_BUILD:
-                if v.endswith("\\"):
-                    mode = 2
-                    del buildlines[:]
-                    buildlines.append(v[:-1])
-                else:
-                    build = parse_buildline([v])
-                    app.builds.append(build)
-                    add_comments('build:' + app.builds[-1].versionCode)
-            elif ftype == TYPE_BUILD_V2:
                 vv = v.split(',')
                 if len(vv) != 2:
                     warn_or_exception(_('Build should have comma-separated '
@@ -1353,7 +1387,9 @@ def parse_txt_metadata(mf, app):
                 del buildlines[:]
                 mode = 3
             elif ftype == TYPE_OBSOLETE:
-                pass        # Just throw it away!
+                warn_or_exception(_("'{field}' in {linedesc} is obsolete, see docs for current fields:")
+                                  .format(field=f, linedesc=linedesc)
+                                  + '\nhttps://f-droid.org/docs/')
             else:
                 warn_or_exception(_("Unrecognised field '{field}' in {linedesc}")
                                   .format(field=f, linedesc=linedesc))
@@ -1549,7 +1585,7 @@ def write_metadata(metadatapath, app):
                           .format(path=metadatapath, formats=', '.join(accepted)))
 
     try:
-        with open(metadatapath, 'w', encoding='utf8') as mf:
+        with open(metadatapath, 'w') as mf:
             if ext == 'txt':
                 return write_txt(mf, app)
             elif ext == 'yml':
