@@ -59,11 +59,14 @@ def _ssh_key_from_debug_keystore(keystore=KEYSTORE_FILE):
                            '-srcstorepass', PASSWORD, '-srckeypass', PASSWORD,
                            '-destkeystore', p12, '-destalias', KEY_ALIAS,
                            '-deststorepass', PASSWORD, '-destkeypass', PASSWORD,
-                           '-deststoretype', 'PKCS12'])
+                           '-deststoretype', 'PKCS12'],
+                          env={'LC_ALL': 'C.UTF-8'})
     subprocess.check_call(['openssl', 'pkcs12', '-in', p12, '-out', key_pem,
-                           '-passin', 'pass:' + PASSWORD, '-passout', 'pass:' + PASSWORD])
+                           '-passin', 'pass:' + PASSWORD, '-passout', 'pass:' + PASSWORD],
+                          env={'LC_ALL': 'C.UTF-8'})
     subprocess.check_call(['openssl', 'rsa', '-in', key_pem, '-out', privkey,
-                           '-passin', 'pass:' + PASSWORD])
+                           '-passin', 'pass:' + PASSWORD],
+                          env={'LC_ALL': 'C.UTF-8'})
     os.remove(key_pem)
     os.remove(p12)
     os.chmod(privkey, 0o600)  # os.umask() should cover this, but just in case
@@ -91,8 +94,12 @@ def main():
                         help=_("Specify which debug keystore file to use."))
     parser.add_argument("--show-secret-var", action="store_true", default=False,
                         help=_("Print the secret variable to the terminal for easy copy/paste"))
+    parser.add_argument("--keep-private-keys", action="store_true", default=False,
+                        help=_("Do not remove the private keys generated from the keystore"))
+    parser.add_argument("--no-deploy", action="store_true", default=False,
+                        help=_("Do not deploy the new files to the repo"))
     parser.add_argument("--file", default='app/build/outputs/apk/*.apk',
-                        help=_('The the file to be included in the repo (path or glob)'))
+                        help=_('The file to be included in the repo (path or glob)'))
     parser.add_argument("--no-checksum", action="store_true", default=False,
                         help=_("Don't use rsync checksums"))
     # TODO add --with-btlog
@@ -283,14 +290,19 @@ Last updated: {date}'''.format(repo_git_base=repo_git_base,
         common.local_rsync(options, repo_basedir + '/metadata/', git_mirror_metadatadir + '/')
         mirror_git_repo.git.add(all=True)
         mirror_git_repo.index.commit("update app metadata")
-        try:
-            subprocess.check_call(['fdroid', 'server', 'update', '--verbose'], cwd=repo_basedir)
-        except subprocess.CalledProcessError:
-            logging.error(_('cannot publish update, did you set the deploy key?')
-                          + '\n' + deploy_key_url)
-            sys.exit(1)
-        if shutil.rmtree.avoids_symlink_attacks:
-            shutil.rmtree(os.path.dirname(ssh_private_key_file))
+
+        if not options.no_deploy:
+            try:
+                subprocess.check_call(['fdroid', 'server', 'update', '--verbose'], cwd=repo_basedir)
+            except subprocess.CalledProcessError:
+                logging.error(_('cannot publish update, did you set the deploy key?')
+                              + '\n' + deploy_key_url)
+                sys.exit(1)
+
+        if not options.keep_private_keys:
+            os.remove(KEYSTORE_FILE)
+            if shutil.rmtree.avoids_symlink_attacks:
+                shutil.rmtree(os.path.dirname(ssh_private_key_file))
 
     else:
         if not os.path.isfile(options.keystore):
