@@ -33,7 +33,7 @@ import zipfile
 from . import _
 from . import common
 from . import metadata
-from .common import FDroidPopen, SdkToolsPopen
+from .common import FDroidPopen
 from .exception import BuildException, FDroidException
 
 config = None
@@ -60,12 +60,12 @@ def key_alias(appid):
         # For this particular app, the key alias is overridden...
         keyalias = config['keyaliases'][appid]
         if keyalias.startswith('@'):
-            m = hashlib.md5()
+            m = hashlib.md5()  # nosec just used to generate a keyalias
             m.update(keyalias[1:].encode('utf-8'))
             keyalias = m.hexdigest()[:8]
         return keyalias
     else:
-        m = hashlib.md5()
+        m = hashlib.md5()  # nosec just used to generate a keyalias
         m.update(appid.encode('utf-8'))
         return m.hexdigest()[:8]
 
@@ -74,7 +74,7 @@ def read_fingerprints_from_keystore():
     """Obtain a dictionary containing all singning-key fingerprints which
     are managed by F-Droid, grouped by appid.
     """
-    env_vars = {'LC_ALL': 'C',
+    env_vars = {'LC_ALL': 'C.UTF-8',
                 'FDROID_KEY_STORE_PASS': config['keystorepass'],
                 'FDROID_KEY_PASS': config['keypass']}
     p = FDroidPopen([config['keytool'], '-list',
@@ -82,7 +82,7 @@ def read_fingerprints_from_keystore():
                      '-storepass:env', 'FDROID_KEY_STORE_PASS'],
                     envs=env_vars, output=False)
     if p.returncode != 0:
-        raise FDroidException('could not read keysotre {}'.format(config['keystore']))
+        raise FDroidException('could not read keystore {}'.format(config['keystore']))
 
     realias = re.compile('Alias name: (?P<alias>.+)\n')
     resha256 = re.compile(r'\s+SHA256: (?P<sha256>[:0-9A-F]{95})\n')
@@ -178,6 +178,7 @@ def main():
     if not os.path.isdir(unsigned_dir):
         logging.warning(_("No unsigned directory - nothing to do"))
         sys.exit(1)
+    binaries_dir = os.path.join(unsigned_dir, 'binaries')
 
     if not os.path.exists(config['keystore']):
         logging.error("Config error - missing '{0}'".format(config['keystore']))
@@ -196,7 +197,7 @@ def main():
     vercodes = common.read_pkg_args(options.appid, True)
     allaliases = []
     for appid in allapps:
-        m = hashlib.md5()
+        m = hashlib.md5()  # nosec just used to generate a keyalias
         m.update(appid.encode('utf-8'))
         keyalias = m.hexdigest()[:8]
         if keyalias in allaliases:
@@ -234,22 +235,27 @@ def main():
             # version if everything checks out.
             # The binary should already have been retrieved during the build
             # process.
-            srcapk = re.sub(r'.apk$', '.binary.apk', apkfile)
 
-            # Compare our unsigned one with the downloaded one...
-            compare_result = common.verify_apks(srcapk, apkfile, tmp_dir)
-            if compare_result:
-                logging.error("...verification failed - publish skipped : "
-                              + compare_result)
+            srcapk = re.sub(r'\.apk$', '.binary.apk', apkfile)
+            srcapk = srcapk.replace(unsigned_dir, binaries_dir)
+
+            if not os.path.isfile(srcapk):
+                logging.error("...reference binary missing - publish skipped: "
+                              "'{refpath}'".format(refpath=srcapk))
             else:
+                # Compare our unsigned one with the downloaded one...
+                compare_result = common.verify_apks(srcapk, apkfile, tmp_dir)
+                if compare_result:
+                    logging.error("...verification failed - publish skipped : "
+                                  "{result}".format(result=compare_result))
+                else:
+                    # Success! So move the downloaded file to the repo, and remove
+                    # our built version.
+                    shutil.move(srcapk, os.path.join(output_dir, apkfilename))
+                    os.remove(apkfile)
 
-                # Success! So move the downloaded file to the repo, and remove
-                # our built version.
-                shutil.move(srcapk, os.path.join(output_dir, apkfilename))
-                os.remove(apkfile)
-
-                publish_source_tarball(apkfilename, unsigned_dir, output_dir)
-                logging.info('Published ' + apkfilename)
+                    publish_source_tarball(apkfilename, unsigned_dir, output_dir)
+                    logging.info('Published ' + apkfilename)
 
         elif apkfile.endswith('.zip'):
 
@@ -274,7 +280,7 @@ def main():
                 signaturefile, signedfile, manifest = signingfiles
 
                 with open(signaturefile, 'rb') as f:
-                    devfp = common.signer_fingerprint_short(f.read())
+                    devfp = common.signer_fingerprint_short(common.get_signature(f.read()))
                 devsigned = '{}_{}_{}.apk'.format(appid, vercode, devfp)
                 devsignedtmp = os.path.join(tmp_dir, devsigned)
                 shutil.copy(apkfile, devsignedtmp)
@@ -301,21 +307,20 @@ def main():
                     # For this particular app, the key alias is overridden...
                     keyalias = config['keyaliases'][appid]
                     if keyalias.startswith('@'):
-                        m = hashlib.md5()
+                        m = hashlib.md5()  # nosec just used to generate a keyalias
                         m.update(keyalias[1:].encode('utf-8'))
                         keyalias = m.hexdigest()[:8]
                 else:
-                    m = hashlib.md5()
+                    m = hashlib.md5()  # nosec just used to generate a keyalias
                     m.update(appid.encode('utf-8'))
                     keyalias = m.hexdigest()[:8]
                 logging.info("Key alias: " + keyalias)
 
                 # See if we already have a key for this application, and
                 # if not generate one...
-                env_vars = {
-                    'FDROID_KEY_STORE_PASS': config['keystorepass'],
-                    'FDROID_KEY_PASS': config['keypass'],
-                }
+                env_vars = {'LC_ALL': 'C.UTF-8',
+                            'FDROID_KEY_STORE_PASS': config['keystorepass'],
+                            'FDROID_KEY_PASS': config['keypass']}
                 p = FDroidPopen([config['keytool'], '-list',
                                  '-alias', keyalias, '-keystore', config['keystore'],
                                  '-storepass:env', 'FDROID_KEY_STORE_PASS'], envs=env_vars)
@@ -350,10 +355,7 @@ def main():
                     raise BuildException(_("Failed to sign application"), p.output)
 
                 # Zipalign it...
-                p = SdkToolsPopen(['zipalign', '-v', '4', apkfile,
-                                   os.path.join(output_dir, apkfilename)])
-                if p.returncode != 0:
-                    raise BuildException(_("Failed to align application"))
+                common._zipalign(apkfile, os.path.join(output_dir, apkfilename))
                 os.remove(apkfile)
 
                 publish_source_tarball(apkfilename, unsigned_dir, output_dir)
