@@ -21,6 +21,7 @@ import re
 import traceback
 from argparse import ArgumentParser
 import logging
+import itertools
 
 from . import _
 from . import common
@@ -33,24 +34,19 @@ options = None
 
 def get_gradle_compile_commands(build):
     compileCommands = ['compile',
-                       'provided'
-                       'apk'
-                       'implementation'
-                       'api'
-                       'compileOnly'
-                       'runtimeOnly',
-                       'releaseCompile'
-                       'releaseProvided'
-                       'releaseApk'
-                       'releaseImplementation'
-                       'releaseApi'
-                       'releaseCompileOnly'
-                       'releaseRuntimeOnly']
+                       'provided',
+                       'apk',
+                       'implementation',
+                       'api',
+                       'compileOnly',
+                       'runtimeOnly']
+    buildTypes = ['', 'release']
+    flavors = ['']
     if build.gradle and build.gradle != ['yes']:
-        compileCommands += [flavor + 'Compile' for flavor in build.gradle]
-        compileCommands += [flavor + 'ReleaseCompile' for flavor in build.gradle]
+        flavors += build.gradle
 
-    return [re.compile(r'\s*' + c, re.IGNORECASE) for c in compileCommands]
+    commands = [''.join(c) for c in itertools.product(flavors, buildTypes, compileCommands)]
+    return [re.compile(r'\s*' + c, re.IGNORECASE) for c in commands]
 
 
 def scan_source(build_dir, build=metadata.Build()):
@@ -79,6 +75,7 @@ def scan_source(build_dir, build=metadata.Build()):
             r'ouya.*sdk',
             r'libspen23',
             r'firebase',
+            r'''["']com.facebook.android['":]''',
         ]
     }
 
@@ -233,7 +230,7 @@ def scan_source(build_dir, build=metadata.Build()):
             elif ext == 'java':
                 if not os.path.isfile(filepath):
                     continue
-                with open(filepath, 'r', encoding='utf8', errors='replace') as f:
+                with open(filepath, 'r', errors='replace') as f:
                     for line in f:
                         if 'DexClassLoader' in line:
                             count += handleproblem('DexClassLoader', path_in_build_dir, filepath)
@@ -242,7 +239,7 @@ def scan_source(build_dir, build=metadata.Build()):
             elif ext == 'gradle':
                 if not os.path.isfile(filepath):
                     continue
-                with open(filepath, 'r', encoding='utf8', errors='replace') as f:
+                with open(filepath, 'r', errors='replace') as f:
                     lines = f.readlines()
                 for i, line in enumerate(lines):
                     if is_used_by_gradle(line):
