@@ -23,21 +23,21 @@ import sys
 import os
 import shutil
 import glob
+import logging
 import re
 import socket
+import warnings
 import zipfile
 import hashlib
-import pickle
+import json
 import time
+import copy
 from datetime import datetime
 from argparse import ArgumentParser
 
 import collections
 from binascii import hexlify
 
-from PIL import Image, PngImagePlugin
-import logging
-
 from . import _
 from . import common
 from . import index
@@ -45,18 +45,22 @@ from . import metadata
 from .common import SdkToolsPopen
 from .exception import BuildException, FDroidException
 
-METADATA_VERSION = 19
+from PIL import Image, PngImagePlugin
+if hasattr(Image, 'DecompressionBombWarning'):
+    warnings.simplefilter('error', Image.DecompressionBombWarning)
+Image.MAX_IMAGE_PIXELS = 0xffffff  # 4096x4096
+
+METADATA_VERSION = 21
 
 # less than the valid range of versionCode, i.e. Java's Integer.MIN_VALUE
 UNSET_VERSION_CODE = -0x100000000
 
-APK_NAME_PAT = re.compile(".*name='([a-zA-Z0-9._]*)'.*")
+APK_NAME_PAT = re.compile(r".*\Wname='([a-zA-Z0-9._]*)'.*")
 APK_VERCODE_PAT = re.compile(".*versionCode='([0-9]*)'.*")
 APK_VERNAME_PAT = re.compile(".*versionName='([^']*)'.*")
 APK_LABEL_ICON_PAT = re.compile(r".*\s+label='(.*)'\s+icon='(.*?)'")
 APK_SDK_VERSION_PAT = re.compile(".*'([0-9]*)'.*")
-APK_PERMISSION_PAT = \
-    re.compile(".*(name='(?P<name>.*?)')(.*maxSdkVersion='(?P<maxSdkVersion>.*?)')?.*")
+APK_PERMISSION_PAT = re.compile(r".*name='([^']*)'(?:.*maxSdkVersion='([^']*)')?.*")
 APK_FEATURE_PAT = re.compile(".*name='([^']*)'.*")
 
 screen_densities = ['65534', '640', '480', '320', '240', '160', '120']
@@ -410,34 +414,31 @@ def resize_all_icons(repodirs):
 
 
 def getsig(apkpath):
-    """ Get the signing certificate of an apk. To get the same md5 has that
-    Android gets, we encode the .RSA certificate in a specific format and pass
-    it hex-encoded to the md5 digest algorithm.
+    """Get the unique ID for the signing certificate of an APK.
+
+    This uses a strange algorithm that was devised at the very
+    beginning of F-Droid.  Since it is only used for checking
+    signature compatibility, it does not matter much that it uses MD5.
+
+    To get the same MD5 has that fdroidclient gets, we encode the .RSA
+    certificate in a specific format and pass it hex-encoded to the
+    md5 digest algorithm.  This is not the same as the standard X.509
+    certificate fingerprint.
 
     :param apkpath: path to the apk
     :returns: A string containing the md5 of the signature of the apk or None
               if an error occurred.
+
     """
 
-    with zipfile.ZipFile(apkpath, 'r') as apk:
-        certs = [n for n in apk.namelist() if common.CERT_PATH_REGEX.match(n)]
-
-        if len(certs) < 1:
-            logging.error(_("No signing certificates found in {path}").format(path=apkpath))
-            return None
-        if len(certs) > 1:
-            logging.error(_("Found multiple signing certificates in {path}").format(path=apkpath))
-            return None
-
-        cert = apk.read(certs[0])
-
-    cert_encoded = common.get_certificate(cert)
-
-    return hashlib.md5(hexlify(cert_encoded)).hexdigest()
+    cert_encoded = common.get_first_signer_certificate(apkpath)
+    if not cert_encoded:
+        return None
+    return hashlib.md5(hexlify(cert_encoded)).hexdigest()  # nosec just used as ID for signing key
 
 
 def get_cache_file():
-    return os.path.join('tmp', 'apkcache')
+    return os.path.join('tmp', 'apkcache.json')
 
 
 def get_cache():
@@ -459,27 +460,46 @@ def get_cache():
     apkcachefile = get_cache_file()
     ada = options.allow_disabled_algorithms or config['allow_disabled_algorithms']
     if not options.clean and os.path.exists(apkcachefile):
-        with open(apkcachefile, 'rb') as cf:
-            apkcache = pickle.load(cf, encoding='utf-8')
+        with open(apkcachefile) as fp:
+            apkcache = json.load(fp, object_pairs_hook=collections.OrderedDict)
         if apkcache.get("METADATA_VERSION") != METADATA_VERSION \
            or apkcache.get('allow_disabled_algorithms') != ada:
-            apkcache = {}
+            apkcache = collections.OrderedDict()
     else:
-        apkcache = {}
+        apkcache = collections.OrderedDict()
 
     apkcache["METADATA_VERSION"] = METADATA_VERSION
     apkcache['allow_disabled_algorithms'] = ada
 
+    for k, v in apkcache.items():
+        if not isinstance(v, dict):
+            continue
+        if 'antiFeatures' in v:
+            v['antiFeatures'] = set(v['antiFeatures'])
+        if 'added' in v:
+            v['added'] = datetime.fromtimestamp(v['added'])
+
     return apkcache
 
 
 def write_cache(apkcache):
+    class Encoder(json.JSONEncoder):
+        def default(self, obj):
+            if isinstance(obj, set):
+                return list(obj)
+            elif isinstance(obj, datetime):
+                return obj.timestamp()
+            return super().default(obj)
+
     apkcachefile = get_cache_file()
     cache_path = os.path.dirname(apkcachefile)
     if not os.path.exists(cache_path):
         os.makedirs(cache_path)
-    with open(apkcachefile, 'wb') as cf:
-        pickle.dump(apkcache, cf)
+    for k, v in apkcache.items():
+        if isinstance(k, bytes):
+            print('BYTES: ' + str(k) + ' ' + str(v))
+    with open(apkcachefile, 'w') as fp:
+        json.dump(apkcache, fp, cls=Encoder, indent=2)
 
 
 def get_icon_bytes(apkzip, iconsrc):
@@ -673,7 +693,7 @@ def _get_localized_dict(app, locale):
 def _set_localized_text_entry(app, locale, key, f):
     limit = config['char_limits'][key]
     localized = _get_localized_dict(app, locale)
-    with open(f) as fp:
+    with open(f, errors='replace') as fp:
         text = fp.read()[:limit]
         if len(text) > 0:
             localized[key] = text
@@ -681,7 +701,7 @@ def _set_localized_text_entry(app, locale, key, f):
 
 def _set_author_entry(app, key, f):
     limit = config['char_limits']['author']
-    with open(f) as fp:
+    with open(f, errors='replace') as fp:
         text = fp.read()[:limit]
         if len(text) > 0:
             app[key] = text
@@ -794,39 +814,30 @@ def copy_triple_t_store_metadata(apps):
 def insert_localized_app_metadata(apps):
     """scans standard locations for graphics and localized text
 
-    Scans for localized description files, store graphics, and
-    screenshot PNG files in statically defined screenshots directory
-    and adds them to the app metadata.  The screenshots and graphic
-    must be PNG or JPEG files ending with ".png", ".jpg", or ".jpeg"
-    and must be in the following layout:
-    # TODO replace these docs with link to All_About_Descriptions_Graphics_and_Screenshots
+    Scans for localized description files, changelogs, store graphics, and
+    screenshots and adds them to the app metadata. Each app's source repo root
+    checked out at /build/<packageName> is scanned at the following standard
+    locations for these files...
 
-    repo/packageName/locale/featureGraphic.png
-    repo/packageName/locale/phoneScreenshots/1.png
-    repo/packageName/locale/phoneScreenshots/2.png
+    metadata/<locale>/
+    fastlane/metadata/android/<locale>/
+    src/<buildFlavor>/fastlane/metadata/android/<locale>/
 
-    The changelog files must be text files named with the versionCode
-    ending with ".txt" and must be in the following layout:
-    https://github.com/fastlane/fastlane/blob/2.28.7/supply/README.md#changelogs-whats-new
+    ...as well as the /metadata/<packageName>/<locale> directory.
 
-    repo/packageName/locale/changelogs/12345.txt
-
-    This will scan the each app's source repo then the metadata/ dir
-    for these standard locations of changelog files.  If it finds
-    them, they will be added to the dict of all packages, with the
-    versions in the metadata/ folder taking precendence over the what
+    If it finds them, they will be added to the dict of all packages, with the
+    versions in the /metadata/ folder taking precendence over the what
     is in the app's source repo.
 
-    Where "packageName" is the app's packageName and "locale" is the locale
-    of the graphics, e.g. what language they are in, using the IETF RFC5646
-    format (en-US, fr-CA, es-MX, etc).
+    The <locale> is the locale of the files supplied in that directory, using
+    the IETF RFC5646 format (e.g. en, en-US, ast, etc).
 
-    This will also scan the app's git for a fastlane folder, and the
-    metadata/ folder and the apps' source repos for standard locations
-    of graphic and screenshot files.  If it finds them, it will copy
-    them into the repo.  The fastlane files follow this pattern:
+    For each <locale> directory, this script searches for the following files
+    in the directory structure as supplied by fastlane. See
     https://github.com/fastlane/fastlane/blob/2.28.7/supply/README.md#images-and-screenshots
 
+    See also our documentation page:
+    https://f-droid.org/en/docs/All_About_Descriptions_Graphics_and_Screenshots/#in-the-apps-build-metadata-in-an-fdroiddata-collection
     """
 
     sourcedirs = glob.glob(os.path.join('build', '[A-Za-z]*', 'src', '[A-Za-z]*', 'fastlane', 'metadata', 'android', '[a-z][a-z]*'))
@@ -900,7 +911,7 @@ def insert_localized_app_metadata(apps):
                             logging.debug('copying ' + f + ' ' + screenshotdestdir)
                             _strip_and_copy_image(f, screenshotdestdir)
 
-    repofiles = sorted(glob.glob(os.path.join('repo', '[A-Za-z]*', '[a-z][a-z][A-Z-.@]*')))
+    repofiles = sorted(glob.glob(os.path.join('repo', '[A-Za-z]*', '[a-z][a-z]*')))
     for d in repofiles:
         if not os.path.isdir(d):
             continue
@@ -947,16 +958,16 @@ def scan_repo_files(apkcache, repodir, knownapks, use_date_from_file=False):
 
     cachechanged = False
     repo_files = []
-    repodir = repodir.encode('utf-8')
+    repodir = repodir.encode()
     for name in os.listdir(repodir):
         file_extension = common.get_file_extension(name)
         if file_extension == 'apk' or file_extension == 'obb':
             continue
         filename = os.path.join(repodir, name)
-        name_utf8 = name.decode('utf-8')
+        name_utf8 = name.decode()
         if filename.endswith(b'_src.tar.gz'):
             logging.debug(_('skipping source tarball: {path}')
-                          .format(path=filename.decode('utf-8')))
+                          .format(path=filename.decode()))
             continue
         if not common.is_repo_file(filename):
             continue
@@ -967,15 +978,8 @@ def scan_repo_files(apkcache, repodir, knownapks, use_date_from_file=False):
 
         shasum = sha256sum(filename)
         usecache = False
-        if name in apkcache:
-            repo_file = apkcache[name]
-            # added time is cached as tuple but used here as datetime instance
-            if 'added' in repo_file:
-                a = repo_file['added']
-                if isinstance(a, datetime):
-                    repo_file['added'] = a
-                else:
-                    repo_file['added'] = datetime(*a[:6])
+        if name_utf8 in apkcache:
+            repo_file = apkcache[name_utf8]
             if repo_file.get('hash') == shasum:
                 logging.debug(_("Reading {apkfilename} from cache")
                               .format(apkfilename=name_utf8))
@@ -1003,10 +1007,10 @@ def scan_repo_files(apkcache, repodir, knownapks, use_date_from_file=False):
                 repo_file['versionCode'] = int(m.group(2))
             srcfilename = name + b'_src.tar.gz'
             if os.path.exists(os.path.join(repodir, srcfilename)):
-                repo_file['srcname'] = srcfilename.decode('utf-8')
+                repo_file['srcname'] = srcfilename.decode()
             repo_file['size'] = stat.st_size
 
-            apkcache[name] = repo_file
+            apkcache[name_utf8] = repo_file
             cachechanged = True
 
         if use_date_from_file:
@@ -1052,6 +1056,13 @@ def scan_apk(apk_file):
     else:
         scan_apk_aapt(apk, apk_file)
 
+    if not common.is_valid_package_name(apk['packageName']):
+        raise BuildException(_("{appid} from {path} is not a valid Java Package Name!")
+                             .format(appid=apk['packageName'], path=apk_file))
+    elif not common.is_strict_application_id(apk['packageName']):
+        logging.warning(_("{appid} from {path} is not a valid Android Package Name!")
+                        .format(appid=apk['packageName'], path=apk_file))
+
     # Get the signature, or rather the signing key fingerprints
     logging.debug('Getting signature of {0}'.format(os.path.basename(apk_file)))
     apk['sig'] = getsig(apk_file)
@@ -1068,8 +1079,6 @@ def scan_apk(apk_file):
     if 'minSdkVersion' not in apk:
         logging.warning("No SDK version information found in {0}".format(apk_file))
         apk['minSdkVersion'] = 3  # aapt defaults to 3 as the min
-    if 'targetSdkVersion' not in apk:
-        apk['targetSdkVersion'] = apk['minSdkVersion']
 
     # Check for known vulnerabilities
     if has_known_vulnerability(apk_file):
@@ -1139,39 +1148,34 @@ def scan_apk_aapt(apk, apkfile):
                 logging.error(line.replace('sdkVersion:', '')
                               + ' is not a valid minSdkVersion!')
             else:
-                apk['minSdkVersion'] = m.group(1)
+                apk['minSdkVersion'] = int(m.group(1))
         elif line.startswith("targetSdkVersion:"):
             m = re.match(APK_SDK_VERSION_PAT, line)
             if m is None:
                 logging.error(line.replace('targetSdkVersion:', '')
                               + ' is not a valid targetSdkVersion!')
             else:
-                apk['targetSdkVersion'] = m.group(1)
+                apk['targetSdkVersion'] = int(m.group(1))
         elif line.startswith("maxSdkVersion:"):
-            apk['maxSdkVersion'] = re.match(APK_SDK_VERSION_PAT, line).group(1)
+            apk['maxSdkVersion'] = int(re.match(APK_SDK_VERSION_PAT, line).group(1))
         elif line.startswith("native-code:"):
             apk['nativecode'] = []
             for arch in line[13:].split(' '):
                 apk['nativecode'].append(arch[1:-1])
         elif line.startswith('uses-permission:'):
-            perm_match = re.match(APK_PERMISSION_PAT, line).groupdict()
-            if perm_match['maxSdkVersion']:
-                perm_match['maxSdkVersion'] = int(perm_match['maxSdkVersion'])
+            perm_match = re.match(APK_PERMISSION_PAT, line).groups()
             permission = UsesPermission(
-                perm_match['name'],
-                perm_match['maxSdkVersion']
+                perm_match[0],  # name
+                None if perm_match[1] is None else int(perm_match[1]),  # maxSdkVersion
             )
-
             apk['uses-permission'].append(permission)
-        elif line.startswith('uses-permission-sdk-23:'):
-            perm_match = re.match(APK_PERMISSION_PAT, line).groupdict()
-            if perm_match['maxSdkVersion']:
-                perm_match['maxSdkVersion'] = int(perm_match['maxSdkVersion'])
-            permission_sdk_23 = UsesPermissionSdk23(
-                perm_match['name'],
-                perm_match['maxSdkVersion']
-            )
 
+        elif line.startswith('uses-permission-sdk-23:'):
+            perm_match = re.match(APK_PERMISSION_PAT, line).groups()
+            permission_sdk_23 = UsesPermissionSdk23(
+                perm_match[0],  # name
+                None if perm_match[1] is None else int(perm_match[1]),  # maxSdkVersion
+            )
             apk['uses-permission-sdk-23'].append(permission_sdk_23)
 
         elif line.startswith('uses-feature:'):
@@ -1189,17 +1193,19 @@ def scan_apk_aapt(apk, apkfile):
 def _sanitize_sdk_version(value):
     """Sanitize the raw values from androguard to handle bad values
 
-    minSdkVersion/targetSdkVersion/maxSdkVersion must be integers,
-    but that doesn't stop devs from doing strange things like
-    setting them using Android XML strings.
+    minSdkVersion/targetSdkVersion/maxSdkVersion must be integers, but
+    that doesn't stop devs from doing strange things like setting them
+    using Android XML strings. This method makes the Androguard output
+    match the output from `aapt dump badging`: bad values are ignored.
 
     https://gitlab.com/souch/SMSbypass/blob/v0.9/app/src/main/AndroidManifest.xml#L29
     https://gitlab.com/souch/SMSbypass/blob/v0.9/app/src/main/res/values/strings.xml#L27
+
     """
     try:
         sdk_version = int(value)
         if sdk_version > 0:
-            return str(sdk_version)  # heinous, but this is still str in the codebase
+            return sdk_version
     except (TypeError, ValueError):
         pass
     return None
@@ -1236,7 +1242,7 @@ def scan_apk_androguard(apk, apkfile):
     androidmanifest_xml = apkobject.xml['AndroidManifest.xml']
     if len(xml.nsmap) > 0:
         # one of them surely will be the Android one, or its corrupt
-        xmlns = '{http://schemas.android.com/apk/res/android}'
+        xmlns = common.XMLNS_ANDROID
     else:
         # strange but sometimes the namespace is blank.  This seems to
         # only happen with the Bromite/Chromium APKs
@@ -1669,6 +1675,9 @@ def apply_info_from_latest_apk(apps, apks):
                     bestver = apk['versionCode']
                     bestapk = apk
 
+                if app.NoSourceSince:
+                    apk['antiFeatures'].add('NoSourceSince')
+
                 if 'added' in apk:
                     if not app.added or apk['added'] < app.added:
                         app.added = apk['added']
@@ -1699,7 +1708,7 @@ def make_categories_txt(repodir, categories):
     catdata = ''
     for cat in sorted(categories):
         catdata += cat + '\n'
-    with open(os.path.join(repodir, 'categories.txt'), 'w', encoding='utf8') as f:
+    with open(os.path.join(repodir, 'categories.txt'), 'w') as f:
         f.write(catdata)
 
 
@@ -1865,13 +1874,8 @@ def main():
                         help=_("Delete APKs and/or OBBs without metadata from the repo"))
     parser.add_argument("-b", "--buildreport", action="store_true", default=False,
                         help=_("Report on build data status"))
-    parser.add_argument("-i", "--interactive", default=False, action="store_true",
-                        help=_("Interactively ask about things that need updating."))
     parser.add_argument("-I", "--icons", action="store_true", default=False,
                         help=_("Resize all the icons exceeding the max pixel size and exit"))
-    parser.add_argument("-e", "--editor", default="/etc/alternatives/editor",
-                        help=_("Specify editor to use in interactive mode. Default is {path}")
-                        .format(path='/etc/alternatives/editor'))
     parser.add_argument("-w", "--wiki", default=False, action="store_true",
                         help=_("Update the wiki"))
     parser.add_argument("--pretty", action="store_true", default=False,
@@ -1892,7 +1896,8 @@ def main():
 
     config = common.read_config(options)
 
-    if not ('jarsigner' in config and 'keytool' in config):
+    if not (('jarsigner' in config or 'apksigner' in config)
+            and 'keytool' in config):
         raise FDroidException(_('Java JDK not found! Install in standard location or set java_paths!'))
 
     repodirs = ['repo']
@@ -2030,7 +2035,9 @@ def main():
     # If there's an archive repo,  make the index for it. We already scanned it
     # earlier on.
     if len(repodirs) > 1:
-        index.make(apps, sortedids, archapks, repodirs[1], True)
+        archived_apps = copy.deepcopy(apps)
+        apply_info_from_latest_apk(archived_apps, archapks)
+        index.make(archived_apps, sortedids, archapks, repodirs[1], True)
 
     git_remote = config.get('binary_transparency_remote')
     if git_remote or os.path.isdir(os.path.join('binary_transparency', '.git')):
@@ -2044,7 +2051,7 @@ def main():
         # Generate latest apps data for widget
         if os.path.exists(os.path.join('stats', 'latestapps.txt')):
             data = ''
-            with open(os.path.join('stats', 'latestapps.txt'), 'r', encoding='utf8') as f:
+            with open(os.path.join('stats', 'latestapps.txt'), 'r') as f:
                 for line in f:
                     appid = line.rstrip()
                     data += appid + "\t"
@@ -2053,7 +2060,7 @@ def main():
                     if app.icon is not None:
                         data += app.icon + "\t"
                     data += app.License + "\n"
-            with open(os.path.join(repodirs[0], 'latestapps.dat'), 'w', encoding='utf8') as f:
+            with open(os.path.join(repodirs[0], 'latestapps.dat'), 'w') as f:
                 f.write(data)
 
     if cachechanged:
